import 'package:diameter/components/dialogs.dart';
import 'package:diameter/config.dart';
import 'package:flutter/material.dart';
import 'package:diameter/components/progress_indicator.dart';
import 'package:diameter/models/basal.dart';
import 'package:diameter/models/basal_profile.dart';
import 'package:diameter/screens/basal/basal_detail.dart';

class BasalListScreen extends StatefulWidget {
  final BasalProfile? basalProfile;

  const BasalListScreen({Key? key, this.basalProfile}) : super(key: key);

  @override
  _BasalListScreenState createState() => _BasalListScreenState();
}

class _BasalListScreenState extends State<BasalListScreen> {
  void refresh({String? message}) {
    setState(() {
      if (widget.basalProfile != null) {
        widget.basalProfile!.basalRates =
            Basal.fetchAllForBasalProfile(widget.basalProfile!);
      }
    });
    setState(() {
      if (message != null) {
        var snackBar = SnackBar(
          content: Text(message),
          duration: const Duration(seconds: 2),
        );
        ScaffoldMessenger.of(context)
          ..removeCurrentSnackBar()
          ..showSnackBar(snackBar);
      }
    });
  }

  void handleEditAction(Basal basal) {
    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => BasalDetailScreen(
          basalProfile: widget.basalProfile!,
          basal: basal,
        ),
      ),
    ).then((message) => refresh(message: message));
  }

  void onDelete(Basal basal) {
    basal.delete().then((_) => refresh(message: 'Basal Rate deleted'));
  }

  void handleDeleteAction(Basal basal) async {
    if (showConfirmationDialogOnDelete) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onDelete(basal),
        message: 'Are you sure you want to delete this Basal Rate?',
      );
    } else {
      onDelete(basal);
    }
  }

  @override
  void initState() {
    super.initState();
    refresh();
  }

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      padding: const EdgeInsets.only(top: 10.0),
      child: Column(
        children: [
          FutureBuilder<List<Basal>>(
            future: widget.basalProfile!.basalRates,
            builder: (context, snapshot) {
              return ViewWithProgressIndicator(
                // TODO: add warning if time period is missing or has multiple rates
                snapshot: snapshot,
                child: snapshot.data == null || snapshot.data!.isEmpty
                    ? const Padding(
                        padding: EdgeInsets.all(10.0),
                        child: Text('No Basal Rates for this Profile'),
                      )
                    : ListBody(
                        children: [
                          DataTable(
                            columnSpacing: 10.0,
                            showCheckboxColumn: false,
                            rows: snapshot.data != null
                                ? snapshot.data!.map((basal) {
                                    return DataRow(
                                      cells: basal.asDataTableCells([
                                        IconButton(
                                            icon: const Icon(Icons.edit),
                                            iconSize: 16.0,
                                            onPressed: () =>
                                                handleEditAction(basal)),
                                        IconButton(
                                          icon: const Icon(Icons.delete),
                                          iconSize: 16.0,
                                          onPressed: () =>
                                              handleDeleteAction(basal),
                                        ),
                                      ]),
                                    );
                                  }).toList()
                                : [],
                            columns: Basal.asDataTableColumns(),
                          ),
                        ],
                      ),
              );
            },
          ),
        ],
      ),
    );
  }
}
