import 'package:diameter/components/dialogs.dart';
import 'package:diameter/components/progress_indicator.dart';
import 'package:diameter/config.dart';
import 'package:diameter/navigation.dart';
import 'package:diameter/screens/meal/meal_category_detail.dart';
import 'package:flutter/material.dart';
import 'package:diameter/models/meal_category.dart';

class MealCategoryListScreen extends StatefulWidget {
  static const String routeName = '/meal-categories';

  const MealCategoryListScreen({Key? key}) : super(key: key);

  @override
  _MealCategoryListScreenState createState() => _MealCategoryListScreenState();
}

class _MealCategoryListScreenState extends State<MealCategoryListScreen> {
  late Future<List<MealCategory>?> _mealCategories;

  void refresh({String? message}) {
    setState(() {
      _mealCategories = MealCategory.fetchAll();
    });
    setState(() {
      if (message != null) {
        var snackBar = SnackBar(
          content: Text(message),
          duration: const Duration(seconds: 2),
        );
        ScaffoldMessenger.of(context)
          ..removeCurrentSnackBar()
          ..showSnackBar(snackBar);
      }
    });
  }

  void onDelete(MealCategory mealCategory) {
    mealCategory
        .delete()
        .then((_) => refresh(message: 'Meal Category deleted'));
  }

  void handleDeleteAction(MealCategory mealCategory) async {
    if (showConfirmationDialogOnDelete) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onDelete(mealCategory),
        message: 'Are you sure you want to delete this Meal Category?',
      );
    } else {
      onDelete(mealCategory);
    }
  }

  @override
  void initState() {
    super.initState();
    refresh();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Meal Categories'),
        actions: <Widget>[
          IconButton(
            onPressed: refresh,
            icon: const Icon(Icons.refresh),
          ),
        ],
      ),
      drawer:
          const Navigation(currentLocation: MealCategoryListScreen.routeName),
      body: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Expanded(
            child: FutureBuilder<List<MealCategory>?>(
              future: _mealCategories,
              builder: (context, snapshot) {
                return ViewWithProgressIndicator(
                  snapshot: snapshot,
                  child: snapshot.data == null || snapshot.data!.isEmpty
                      ? Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: const [
                            Padding(
                              padding: EdgeInsets.all(10.0),
                              child: Text('No Meal Categories'),
                            ),
                          ],
                        )
                      : ListView.builder(
                          padding: const EdgeInsets.only(top: 10.0),
                          itemCount:
                              snapshot.data != null ? snapshot.data!.length : 0,
                          itemBuilder: (context, index) {
                            final mealCategory = snapshot.data![index];
                            return ListTile(
                              onTap: () {
                                Navigator.push(
                                  context,
                                  MaterialPageRoute(
                                    builder: (context) =>
                                        MealCategoryDetailScreen(
                                      mealCategory: mealCategory,
                                    ),
                                  ),
                                ).then((message) => refresh(message: message));
                              },
                              title: Text(mealCategory.value),
                              subtitle: Text(mealCategory.notes ?? ''),
                              trailing: Row(
                                mainAxisSize: MainAxisSize.min,
                                children: [
                                  IconButton(
                                    icon: const Icon(
                                      Icons.delete,
                                      color: Colors.blue,
                                    ),
                                    onPressed: () =>
                                        handleDeleteAction(mealCategory),
                                  ),
                                ],
                              ),
                            );
                          }),
                );
              },
            ),
          ),
        ],
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => const MealCategoryDetailScreen(),
            ),
          ).then((message) => refresh(message: message));
        },
        child: const Icon(Icons.add),
      ),
    );
  }
}
